require "prefabutil"

local assets =
{
    Asset("ANIM", "anim/statuerobobee" .. ROBOBEE_SKIN .. ".zip"),
}

local prefabs =
{
	"statuerobobee_blueprint",
}

local PLACER_SCALE = 1.55

local function UpdateSwapBee(inst)
	if inst.components.childspawner and inst.components.childspawner.numchildrenoutside <= 0 then
		inst.SoundEmitter:PlaySound("robobeesounds/robobeesounds/sleep", "zzz") 
		inst.AnimState:Show("SWAP_ROBOBEE") 
		if inst.components.sanityaura then
			inst.components.sanityaura.aura = TUNING.SANITYAURA_TINY/2
		end
	else 
		inst.AnimState:Hide("SWAP_ROBOBEE") 
		inst.SoundEmitter:KillSound("zzz")
		if inst.components.sanityaura then
			inst.components.sanityaura.aura = 0
		end
	end
end

local function UpdateContainerTable(inst)
	local inv = inst.components.container
	
	local allslots = {1, 2, 3, 4, 5, 6, 7, 8, 9}
	
	for k, v in pairs (allslots) do
		inst.AnimState:OverrideSymbol(tostring(k), "statuerobobee" .. ROBOBEE_SKIN, "blank_frame")
	end
	
	for k, v in pairs(inv.slots) do
		if v ~= nil then
			inst.AnimState:OverrideSymbol(tostring(k), "statuerobobee" .. ROBOBEE_SKIN, tostring(k))
		end
	end
	
end

local function OnRobobeeSpawned(inst, child)
    if inst.components.childspawner and child.components.follower and child.components.follower.leader ~= inst then
        child.components.follower:SetLeader(inst)
		inst.AnimState:Hide("SWAP_ROBOBEE")
		if STATUEROBOBEE_CONTAINER == "chest" then
			inst.MiniMapEntity:SetIcon("statuerobobee_map" .. ROBOBEE_SKIN .. ".tex")
		else
			inst.MiniMapEntity:SetIcon("statuerobobee_map_icebox" .. ROBOBEE_SKIN .. ".tex")
		end
		--print("Spawned Bee")
    end
end

local function OnRobobeeBackHome(inst, child)
	if child and child.components.inventory then
		inst.AnimState:Show("SWAP_ROBOBEE")
		
		if child.components.inventory:NumItems() ~= 0 then
			-- transfer items to base
			local inv = inst.components.container

			if inv and inv:IsOpen() then
				inv:Close()
			end
		
			for k,v in pairs(child.components.inventory.itemslots) do
				inst.components.container:GiveItem(child.components.inventory:RemoveItemBySlot(k))
			end
		
			UpdateContainerTable(inst)
		end
		
		if STATUEROBOBEE_CONTAINER == "chest" then
			inst.MiniMapEntity:SetIcon("statuerobobee_map_full" .. ROBOBEE_SKIN .. ".tex")
		else
			inst.MiniMapEntity:SetIcon("statuerobobee_map_full_icebox" .. ROBOBEE_SKIN .. ".tex")
		end
		--print("Bee Going Home")
	end
end

local function OnDestroyed(inst, worker)
    if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
        inst.components.burnable:Extinguish()
    end
	if inst.components.childspawner and inst.components.childspawner.numchildrenoutside <= 0 then
		inst.components.childspawner:ReleaseAllChildren()
	end
	
	if inst.components.childspawner then
		for k,v in pairs(inst.components.childspawner.childrenoutside) do
			if v and v:IsValid() then
				if not v.components.health then
					v:AddComponent("health")
					v.components.health:DoDelta(-1000)
					--v:PushEvent("death")
				end 
			end
		end
	end
	
    inst.components.lootdropper:DropLoot()
	
	if inst.components.container then
		inst.components.container:DropEverything()
	end
	
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
	if STATUEROBOBEE_CONTAINER == "chest" then
		inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")	
	else
		inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
	end
    inst:Remove()
end

local function OnHit(inst, worker, workleft)

	if not inst:HasTag("burnt") then
        inst.AnimState:PlayAnimation("hit")
        inst.AnimState:PushAnimation("idle", true)
    end
	
	local inv = inst.components.container

	if inv and inv:IsOpen() then
		inv:Close()
	end
	
	if worker and worker.prefab and (worker.prefab == "pigman_royalguard" or worker.prefab == "pigman_royalguard_2") then
		inst.components.workable.workleft = inst.components.workable.workleft - 1
	end
end

local function PickableCheck(v, target)
	return target 
	and target.components.pickable 
	and target.components.pickable:CanBePicked() 
	and target.components.pickable.product ~= nil 
	and target.components.pickable.product == v.prefab 
	and v.components.stackable 
	and not v.components.stackable:IsFull() 
	and ((target.components.pickable.jostlepick == nil or target.components.pickable.jostlepick == false) and target.components.pickable.numtoharvest <= v.components.stackable:RoomLeft() 
	or target.components.pickable.jostlepick == true and 1 <= v.components.stackable:RoomLeft())
end

local function iswinter()
	return GetSeasonManager() and GetSeasonManager():IsWinter() or false
end

local function iscold()
	return GetSeasonManager() and GetSeasonManager():GetCurrentTemperature() <= 0 or false
end

local function isnotcave()
	return GetWorld() and not GetWorld():IsCave() and true or false
end

local function checkharvestconfiguration(target, prefab)
	return ROBOBEE_HARVEST == 1 and ((iswinter() and iscold() and isnotcave() and prefab == "beebox") or target.produce == target.maxproduce) and true
	or ROBOBEE_HARVEST == 2 and ((iswinter() and iscold() and isnotcave() and prefab == "beebox") or target.produce >= math.ceil(target.maxproduce*0.5)) and true
	or ROBOBEE_HARVEST == 3 and target.produce > 0 and true
	or false
end

local function HarvestableCheck(v, target)
	return target 
	and target.components.harvestable 
	and target.components.harvestable:CanBeHarvested()
	and target.components.harvestable.produce ~= nil 
	and checkharvestconfiguration(target.components.harvestable, target.prefab) == true
	and tostring(target.components.harvestable.product) == v.prefab 
	and v.components.stackable 
	and not v.components.stackable:IsFull() 
	and target.components.harvestable.produce <= v.components.stackable:RoomLeft()
end

local function DryerCheck(v, target)
	return target 
	and target.components.dryer 
	and target.components.dryer:IsDone()
	and target.components.dryer.product ~= nil  
	and target.components.dryer.product == v.prefab 
	and v.components.stackable 
	and not v.components.stackable:IsFull() 
	and 1 <= v.components.stackable:RoomLeft()
end

local function CropCheck(v, target)
	return target 
	and target.components.crop 
	and target.components.crop:IsReadyForHarvest()
	and target.components.crop.product_prefab ~= nil  
	and target.components.crop.product_prefab == v.prefab 
	and v.components.stackable 
	and not v.components.stackable:IsFull() 
	and 1 <= v.components.stackable:RoomLeft()
end

local function Crop_LegionCheck(v, target)
	return target 
	and target.components.crop_legion 
	and target.components.crop_legion:IsReadyForHarvest()
	and target.components.crop_legion.product_prefab ~= nil  
	and target.components.crop_legion.product_prefab == v.prefab 
	and v.components.stackable 
	and not v.components.stackable:IsFull() 
	and target.components.crop_legion.numfruit <= v.components.stackable:RoomLeft()
end

local function CheckInvForViableCheck(inst, target)
	for k,v in pairs(inst.components.container.slots) do
		if target and not target.components.pickable and target.prefab and v and v.prefab and v.prefab == target.prefab and not v.components.stackable:IsFull() and (not target.components.burnable or target.components.burnable and not target.components.burnable:IsBurning()) or (PickableCheck(v, target) or HarvestableCheck(v, target) or DryerCheck(v, target) or CropCheck(v, target) or Crop_LegionCheck(v, target)) then
			return true
		end
	end
end

local function inherentexclusioncheck(target)
	return not target.components.harvestable and true
	or target.components.harvestable:CanBeHarvested() and target.components.harvestable.produce ~= nil and checkharvestconfiguration(target.components.harvestable, target.prefab)
end

local function CheckAreaAndSpawnBee(inst)
	if inst.components.childspawner and inst.components.childspawner.numchildrenoutside <= 0 then
		
		local target = FindEntity(inst, 15, function(item)
			return item and item:IsValid() and item.prefab and CheckInvForViableCheck(inst, item) or not inst.components.container:IsFull() and inherentexclusioncheck(item) or false
		end, nil, STATUEROBOBEE_EXCLUDETAGS, STATUEROBOBEE_INCLUDETAGS)
		
		local cantakeitem = nil
		
		local truetarget = target ~= nil and target.components.inventoryitem and target or nil
		local trueproduct = (target ~= nil and target.components.pickable and target.components.pickable.product) or 
			(target ~= nil and target.components.harvestable and target.components.harvestable.product ~= nil and tostring(target.components.harvestable.product)) or
			(target ~= nil and target.components.dryer and target.components.dryer.product) or
			(target ~= nil and target.components.crop_legion and target.components.crop_legion.product_prefab) or
			(target ~= nil and target.components.crop and target.components.crop.product_prefab) or nil
		
		-- search for the item in base's inventory (this doesn't activate if inv is empty)
		for k,v in pairs(inst.components.container.slots) do
			if trueproduct == nil and truetarget and truetarget.components and truetarget:IsValid() and v and v.prefab and truetarget.prefab and truetarget.prefab == v.prefab then
				cantakeitem = true
				--break
				print ("Can take item (inventoryitem)")
				
			elseif truetarget == nil and trueproduct then
				if v and v.prefab == trueproduct then
					cantakeitem = true
					--break
					--print ("Can take item (pickable)")
					--print("Prefab is: " .. trueproduct)
				end
			else
				--print ("Cannot take item or no item found!")
			end
		end
		
		-- below function goes through even if the base's inventory is completely empty
		if inst.components.container and not inst.components.container:IsFull() then
			cantakeitem = true
			--print ("Free space in base detected.")
		end
		
		if target and cantakeitem and target.robobee_picker == nil then
			--print("Releasing bee...")
			if inst.components.childspawner.numchildrenoutside <= 0 then
				inst.components.childspawner:ReleaseAllChildren()
			end
			inst.passtargettobee = target -- let's try passing the target
		else
			if inst.passtargettobee ~= nil then
				inst.passtargettobee = nil -- clear the variable if nothing was found
			end
		end
	end
end

local function onopen(inst)
    if not inst:HasTag("burnt") then
        inst.AnimState:OverrideSymbol("chest", "statuerobobee" .. ROBOBEE_SKIN, STATUEROBOBEE_CONTAINER .. "_lid_open")
		if STATUEROBOBEE_CONTAINER == "chest" then
			inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
		else
			inst.SoundEmitter:PlaySound("dontstarve/common/craftable/icebox_open")
		end
    end
end 

local function onclose(inst)
    if not inst:HasTag("burnt") then
        inst.AnimState:OverrideSymbol("chest", "statuerobobee" .. ROBOBEE_SKIN, STATUEROBOBEE_CONTAINER)
		if STATUEROBOBEE_CONTAINER == "chest" then
			inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
		else
			inst.SoundEmitter:PlaySound("dontstarve/common/craftable/icebox_close")
		end
    end
	UpdateContainerTable(inst)
end

local function OnBuilt(inst)
	inst.AnimState:PlayAnimation("building")
    inst.AnimState:PushAnimation("idle", true)
end

local function UpdateMapIcon(inst)
	if inst.components.childspawner and inst.components.childspawner.numchildrenoutside <= 0 then
		if STATUEROBOBEE_CONTAINER == "chest" then
			inst.MiniMapEntity:SetIcon("statuerobobee_map_full" .. ROBOBEE_SKIN .. ".tex")
		else
			inst.MiniMapEntity:SetIcon("statuerobobee_map_full_icebox" .. ROBOBEE_SKIN .. ".tex")
		end
	else
		if STATUEROBOBEE_CONTAINER == "chest" then
			inst.MiniMapEntity:SetIcon("statuerobobee_map" .. ROBOBEE_SKIN .. ".tex")
		else
			inst.MiniMapEntity:SetIcon("statuerobobee_map_icebox" .. ROBOBEE_SKIN .. ".tex")
		end
	end
end

local function MakeSureHasChild(inst)
	-- In case the robobee bugged out for some reason and is not near the base while loading, remove the bee, add a new bee to base
	if inst.components.childspawner and inst.components.childspawner.numchildrenoutside > 0 then
		local bee = FindEntity(inst, 20, nil, nil, {"player"}, {"robobee"})
		if bee == nil then
			local childtokill = inst.components.childspawner.childrenoutside[1]
			if childtokill ~= nil then
				childtokill:Remove()
				table.remove(inst.components.childspawner.childrenoutside, childtokill)
			end
			inst.components.childspawner.childrenoutside = {}
			inst.components.childspawner.numchildrenoutside = 0
			inst.components.childspawner.childreninside = 1
			UpdateSwapBee(inst)
		end
	-- In case there's no bee inside nor outside, force add it
	elseif inst.components.childspawner and inst.components.childspawner.numchildrenoutside <= 0 and inst.components.childspawner.childreninside <= 0 then
		inst.components.childspawner.numchildrenoutside = 0
		inst.components.childspawner.childreninside = 1
		UpdateSwapBee(inst)
	end
end

local function getstatus(inst)
    return inst.components.childspawner and inst.components.childspawner.numchildrenoutside <= 0 and "BEEINSIDE" or nil
end

local slotpos = {}

for y = 2, 0, -1 do
	for x = 0, 2 do
		table.insert(slotpos, Vector3(80*x-80*2+80, 80*y-80*2+80,0))
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    MakeObstaclePhysics(inst, .5)

    inst.MiniMapEntity:SetPriority(5)
	if STATUEROBOBEE_CONTAINER == "chest" then
		inst.MiniMapEntity:SetIcon("statuerobobee_map" .. ROBOBEE_SKIN .. ".tex")
	else
		inst.MiniMapEntity:SetIcon("statuerobobee_map_icebox" .. ROBOBEE_SKIN .. ".tex")
	end

    inst.AnimState:SetBank("statuerobobee" .. ROBOBEE_SKIN)
    inst.AnimState:SetBuild("statuerobobee" .. ROBOBEE_SKIN)
    inst.AnimState:PlayAnimation("idle", true)
	inst.AnimState:OverrideSymbol("chest", "statuerobobee" .. ROBOBEE_SKIN, STATUEROBOBEE_CONTAINER)

    inst:AddTag("statue")
	inst:AddTag("statuerobobee")
	if STATUEROBOBEE_CONTAINER == "chest" then
		inst:AddTag("chest")
	else
		inst:AddTag("fridge")
	end

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = getstatus

    inst:AddComponent("leader")

    inst:AddComponent("childspawner")
    inst.components.childspawner.childname = "robobee"
    inst.components.childspawner:SetMaxChildren(1)
	inst.components.childspawner:SetSpawnedFn(OnRobobeeSpawned)
	inst.components.childspawner:SetGoHomeFn(OnRobobeeBackHome)
	inst.components.childspawner:StopRegen()
	
	inst:AddComponent("container")
	inst.components.container:SetNumSlots(#slotpos)
	
	inst.components.container.onopenfn = onopen
    inst.components.container.onclosefn = onclose
	
    inst.components.container.widgetslotpos = slotpos
	inst.components.container.widgetanimbank = "ui_chest_3x3"
	inst.components.container.widgetanimbuild = "ui_chest_3x3"
	inst.components.container.widgetpos = Vector3(0,200,0)
	inst.components.container.side_align_tip = 160

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(6)
    inst.components.workable:SetOnFinishCallback(OnDestroyed)
    inst.components.workable:SetOnWorkCallback(OnHit)
	inst:ListenForEvent("onbuilt", OnBuilt)

    inst:AddComponent("lootdropper")
	
	inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = 0
	
	inst.updateswapbee = UpdateSwapBee
	inst.checkarea = CheckAreaAndSpawnBee
	
	inst:DoPeriodicTask(3, function(inst) CheckAreaAndSpawnBee(inst) end)
	
	inst:DoPeriodicTask(100*FRAMES, function(inst)
		UpdateSwapBee(inst)
	end)
	
	inst:DoTaskInTime(0, function(inst) 
		UpdateContainerTable(inst) 
		UpdateSwapBee(inst)
		UpdateMapIcon(inst)
		MakeSureHasChild(inst)
	end)
	
	return inst
	
end
    return Prefab("common/statuerobobee", fn, assets, prefabs),
	 MakePlacer("common/statuerobobee_placer", "firefighter_placement", "firefighter_placement", "idle", true, nil, nil, PLACER_SCALE)